import { RequestEvent, Handle } from '@sveltejs/kit';
import { createClient } from 'redis';
import { Firestore } from 'firebase-admin/firestore';

interface KeyInfoData {
    user: string;
    name: string;
    description: string;
    expires: Date | null;
    permissions: string[];
}
interface KeyInfo extends KeyInfoData {
    hash: string;
}

interface KeyStore {
    put(hash: string, info: KeyInfoData): Promise<void>;
    get(hash: string): Promise<KeyInfoData | null>;
    del(hash: string): Promise<void>;
    list(user: string): Promise<KeyInfo[]>;
}

/**
 * Number of seconds per second
 */
declare const SECOND = 1;
/**
 * Number of seconds per minute
 */
declare const MINUTE = 60;
/**
 * Number of seconds per hour
 */
declare const HOUR = 3600;
/**
 * Number of seconds per day
 */
declare const DAY = 86400;
/**
 * Number of seconds per week
 */
declare const WEEK = 604800;
interface Refill {
    readonly rate: number;
    readonly size: number;
}
/**
 * Parses a string representation of a refill parameter
 *
 * @param value The refill parameter string representation
 * @returns A refill object
 */
declare function parseRefill(value: string): {
    rate: number;
    size: number;
};

interface BucketResult {
    allowed: boolean;
    limit: number;
    remaining: number;
    reset: number;
    policy: string;
}
interface TokenBuckets {
    consume(key: string, refill: Refill, count?: number): Promise<BucketResult>;
}
declare abstract class TokenBucket {
    protected abstract consume(key: string, refill: Refill, count?: number): Promise<BucketResult>;
    protected apply(refill: Refill, count: number, allowed: boolean, tokens: number): {
        allowed: boolean;
        limit: number;
        remaining: number;
        reset: number;
        policy: string;
    };
}

type Input = Parameters<Handle>[0];
type CustomFn = (event: RequestEvent, key: string | null) => Promise<string | null>;
interface BaseParam {
    searchParam?: string;
    httpHeader?: string;
    cookie?: string;
    custom?: CustomFn;
    key_length?: number;
}
interface SearchParam extends BaseParam {
    searchParam: string;
}
interface HttpHeader extends BaseParam {
    httpHeader: string;
}
interface Cookie extends BaseParam {
    cookie: string;
}
interface Custom extends BaseParam {
    custom: CustomFn;
}
type Options = SearchParam | HttpHeader | Cookie | Custom;
declare class ApiKeys {
    private readonly storage;
    private readonly buckets;
    private readonly options;
    constructor(storage: KeyStore, buckets: TokenBuckets, options?: Options);
    private hashFromBytes;
    generate(info: KeyInfoData): Promise<{
        key: string;
        hash: string;
    }>;
    validate(key: string | null): Promise<{
        user: string;
        name: string;
        description: string;
        expires: Date | null;
        permissions: string[];
        hash: string;
    } | null>;
    remove(hash: string): Promise<void>;
    retrieve(user: string): Promise<KeyInfo[]>;
    extract(event: RequestEvent): Promise<string | null>;
    handle: (input: Input) => Promise<Response>;
}

declare class InMemoryTokenBucket extends TokenBucket {
    protected readonly buckets: Map<string, {
        tokens: number;
        updated: number;
    }>;
    consume(key: string, refill: Refill, count?: number): Promise<{
        allowed: boolean;
        limit: number;
        remaining: number;
        reset: number;
        policy: string;
    }>;
}

type Redis$1 = ReturnType<typeof createClient>;
declare class RedisTokenBucket extends TokenBucket {
    private readonly redis;
    private readonly prefix;
    private readonly module;
    private readonly fname;
    private constructor();
    consume(key: string, refill: Refill, count?: number): Promise<{
        allowed: boolean;
        limit: number;
        remaining: number;
        reset: number;
        policy: string;
    }>;
    static create(redis: Redis$1, prefix?: string, module?: string, fname?: string): Promise<RedisTokenBucket>;
}

declare class LruCacheKeyStore implements KeyStore {
    private readonly store;
    private cache;
    constructor(store: KeyStore, maxSize?: number, // default capacity
    ttl?: number);
    put(hash: string, info: KeyInfoData): Promise<void>;
    get(hash: string): Promise<KeyInfoData | null>;
    del(hash: string): Promise<void>;
    list(user: string): Promise<KeyInfo[]>;
}

declare class FirestoreKeyStore implements KeyStore {
    private readonly firestore;
    private readonly name;
    private readonly collection;
    constructor(firestore: Firestore, name?: string);
    put(hash: string, info: KeyInfoData): Promise<void>;
    get(hash: string): Promise<KeyInfo | null>;
    del(hash: string): Promise<void>;
    list(user: string): Promise<KeyInfo[]>;
}

declare class InMemoryKeyStore implements KeyStore {
    private keys;
    put(hash: string, info: KeyInfoData): Promise<void>;
    get(hash: string): Promise<KeyInfoData | null>;
    del(hash: string): Promise<void>;
    list(user: string): Promise<{
        user: string;
        name: string;
        description: string;
        expires: Date | null;
        permissions: string[];
        hash: string;
    }[]>;
}

type Redis = ReturnType<typeof createClient>;
declare class RedisKeyStore implements KeyStore {
    private readonly redis;
    private readonly prefix;
    static create(redis: Redis, prefix?: string): Promise<RedisKeyStore>;
    private constructor();
    put(hash: string, info: KeyInfoData): Promise<void>;
    get(hash: string): Promise<KeyInfoData | null>;
    del(hash: string): Promise<void>;
    list(user: string): Promise<{
        hash: string;
        user: string;
        name: string;
        description: string;
        permissions: string[];
        expires: Date | null;
    }[]>;
}

declare class Api {
    private readonly event;
    private readonly bucket;
    readonly key: string | null;
    readonly info: KeyInfo | null;
    private _name;
    private _cost;
    private _anon;
    private _has;
    private _any;
    private _all;
    /**
     * Number of seconds per second
     */
    readonly SECOND = 1;
    /**
     * Number of seconds per minute
     */
    readonly MINUTE = 60;
    /**
     * Number of seconds per hour
     */
    readonly HOUR = 3600;
    /**
     * Number of seconds per day
     */
    readonly DAY = 86400;
    /**
     * Number of seconds per week
     */
    readonly WEEK = 604800;
    constructor(event: RequestEvent, bucket: TokenBuckets, key: string | null, info: KeyInfo | null);
    /**
     * Allow anonymous requests without any API key
     */
    anonymous(): this;
    /**
     * Set the token bucket name to use for rate limiting.
     * Each named token bucket maintains a separate rate-limit.
     * Without a name, a single rate limit will apply across all calls.
     *
     * @param {string} name - the name of the token bucket
     */
    name(name: string): this;
    /**
     * Set the token cost to use when rate-limiting.
     * This is the number of tokens consumed when called.
     *
     * @param {number} cost - the number of tokens to consume
     */
    cost(cost: number): this;
    /**
     * Requires that the API Key has the given permission.
     *
     * @param {string} permission - the permission
     */
    has(permission: string): this;
    /**
     * Requires that the API Key has _any_ of the specified permissions.
     * This allows you to check for single or all permissions, such as
     * `read:my-project` and `read:*` (for all projects)
     *
     * @param {string[]} permissions - the permissions
     */
    any(permissions: string[]): this;
    /**
     * Requires that the API Key has _all_ of the specified permissions.
     * This allows you to check that the caller has permissions on a hierarchy
     * of entities.
     *
     * @param {string[]} permissions - the permissions
     */
    all(permissions: string[]): this;
    /**
     * Verify the call is allowed and is within the rate limit supplied.
     * Rate-limiting http headers are added to the response whether the
     * request is limited or not.
     *
     * If the request should be denied an error will be thrown and handled
     * by SvelteKit otherwise you can continue with generating the response.
     *
     * @param {Object} refill - the token bucket refill parameters
     * @param {number} refill.rate - the refill rate in tokens per second
     * @param {number} refill.size - the size / capacity of the token bucket
     * @throws {HttpError} Will throw a 401 error if the request doesn't include an api key and the call is not set to be anonymous
     * @throws {HttpError} Will throw a 403 error if the provided api key is invalid
     * @throws {HttpError} Will throw a 403 error if the provided api key has expired
     * @throws {HttpError} Will throw a 403 error if the provided api key lacks the required permissions
     * @throws {HttpError} Will throw a 429 error if the rate-limit is hit
     */
    limit(refill: Refill): Promise<BucketResult>;
}

interface ApiLocals {
    api: Api;
}
declare global {
    namespace App {
        interface Locals extends ApiLocals {
        }
    }
}

export { ApiKeys, type ApiLocals, type BucketResult, type Cookie, type Custom, DAY, FirestoreKeyStore, HOUR, type HttpHeader, InMemoryKeyStore, InMemoryTokenBucket, type KeyInfo, type KeyInfoData, type KeyStore, LruCacheKeyStore, MINUTE, type Options, RedisKeyStore, RedisTokenBucket, type Refill, SECOND, type SearchParam, TokenBucket, type TokenBuckets, WEEK, parseRefill };
